OEP: 1
Title: Core Component Design of OWA's Env - Callable, Listener, and Runnable
Author: Open World Agents Team
Status: Final
Type: Standards Track
Created: 01-Feb-2025
Post-History: 07-Jun-2025

Abstract
========

This OEP describes the core component design of Open World Agents' Environment (Env) framework, specifically the three fundamental building blocks: Callables, Listeners, and Runnables. Unlike traditional synchronous step-based environments, these components enable an asynchronous, event-driven architecture that allows agents to react in real-time to environmental events. This design provides the foundation for building high-performance, real-time agent environments.

Motivation
==========

Traditional environmental interfaces like gymnasium.Env fall short when building real-time, real-world agents. They rely on synchronous steps (env.step(), env.reset()), which assume agents have infinite time to process actions. This is unrealistic for agents that need to react instantly in dynamic environments such as desktop automation, game AI, or real-world robotics.

The need for a new paradigm arose from several limitations of existing frameworks:

1. **Synchronous Bottlenecks**: Step-based environments create artificial synchronization points that don't exist in real-world scenarios
2. **Limited Event Handling**: No native support for asynchronous event streams from multiple sources
3. **Rigid Architecture**: Difficulty in extending functionality without modifying core framework code
4. **Performance Constraints**: Inability to achieve sub-30ms latency requirements for real-time applications

OWA's Env addresses these limitations by providing an asynchronous, event-driven architecture that mirrors real-world interactions.

Rationale
=========

The design philosophy centers on three key principles:

**Real-time Performance**: The framework prioritizes low-latency interactions with a target of under 30ms for critical operations. This enables human-level or superhuman reaction times.

**Asynchronous by Design**: Instead of artificial step synchronization, the framework uses event-driven patterns that allow multiple concurrent processes to operate independently.

**Modular Component Design**: The three-component architecture provides clear separation of concerns while maintaining simplicity and flexibility, enabling different interaction patterns within a unified framework.

The three-component architecture (Callables, Listeners, Runnables) was chosen to provide clear separation of concerns while maintaining simplicity and flexibility.

Specification
=============

## Core Components

### 1. Callable
Functions that are actively called to perform actions or retrieve state information.

**Interface**:
```python
class CallableMixin(ABC):
    @abstractmethod
    def __call__(self): ...
```

**Usage Pattern**:
- User initiates the call
- Synchronous execution
- Returns immediate results
- Example: `CALLABLES["mouse.click"](x, y)`

### 2. Listener
Components that respond to events and execute user-provided callbacks.

**Interface**:
```python
class ListenerMixin(RunnableMixin):
    def configure(self, *args, callback: Callable, **kwargs) -> Self: ...

    @abstractmethod
    def loop(self, stop_event, callback): ...
```

**Usage Pattern**:
- Environment initiates the call to user callback
- Asynchronous execution in separate thread/process
- Event-driven activation
- Example: Keyboard listener calling callback on key press

### 3. Runnable
Background processes that can be started, stopped, and managed.

**Interface**:
```python
class RunnableMixin(ABC):
    @abstractmethod
    def start(self): ...

    @abstractmethod
    def stop(self): ...

    @abstractmethod
    def join(self): ...

    @abstractmethod
    def loop(self, stop_event): ...
```

**Usage Pattern**:
- Long-running background processes
- Lifecycle management (start/stop/join)
- Context manager support via `.session`

## Component Relationships

The three components work together to provide different interaction patterns:

1. **Callable ↔ User**: Direct, synchronous interaction for immediate actions
2. **Listener → User**: Asynchronous, event-driven callbacks from environment to user
3. **Runnable**: Independent background processes with managed lifecycle

This separation allows for:
- **Clear Mental Models**: Each component has a distinct purpose and usage pattern
- **Flexible Composition**: Components can be combined to create complex behaviors
- **Performance Optimization**: Each component type can be optimized for its specific use case

Evolution from Initial Design
============================

## Original Concepts (env.py)

The initial design in `oep-0001/env.py` introduced the fundamental concepts:

```python
class Listener(Node):
    """Listener objects that listen to input"""

class Registry:
    def register(self, name: str):
        def decorator(cls):
            self._registry[name] = cls
            return cls
        return decorator

CALLABLES = Registry()
LISTENERS = Registry()
```

**Key Ideas**:
- Separation between Callables (user-initiated) and Listeners (environment-initiated)
- Node-based inheritance for Listeners
- Basic component organization concepts

## Executor Design Evolution (executor_design.py)

The initial executor concept evolved into the current Runnable system:

**Original Concept**:
```python
class TaskDefinition:
    configure_fn: Callable
    loop_fn: Callable
    cleanup_fn: Callable

class Executor:
    def execute(configure_fn, loop_fn, cleanup_fn): ...
```

**Current Implementation**:
```python
class RunnableMixin:
    def configure(self, *args, **kwargs): ...
    def start(self): ...
    def stop(self): ...
    def join(self): ...

    @abstractmethod
    def loop(self, stop_event): ...
```

**Evolution Rationale**:
- Simplified from separate executor to integrated lifecycle management
- Unified interface for both thread and process-based execution
- Context manager support for easier resource management

## Current Implementation Improvements

### Enhanced Type Safety
- Generic registry types: `Registry[T]`
- Proper type annotations throughout
- Abstract base classes for clear interfaces

### Improved Lifecycle Management
- Context manager support via `.session`
- Automatic resource cleanup
- Thread and process variants

### Component Interface Maturity
- Standardized method signatures
- Consistent lifecycle management
- Better error handling and validation

### Performance Optimizations
- Sub-30ms latency targets
- Hardware-accelerated screen capture
- Efficient event handling

Backwards Compatibility
=======================

This OEP represents the foundational design that has been implemented and is currently in use. As this is the initial design specification, there are no backwards compatibility concerns. Future OEPs that modify this design must consider compatibility with existing plugins and user code.

Security Implications
=====================

The component design has several security considerations:

1. **Resource Management**: Improper Runnable implementations could cause resource leaks or consume excessive system resources
2. **Callback Execution**: Listener callbacks execute user-provided code that could be malicious or poorly implemented
3. **Thread Safety**: Concurrent access to shared resources through Callables requires careful synchronization

Mitigation strategies include proper resource cleanup, callback validation, and thread-safe implementations.

How to Teach This
=================

The three-component model maps to familiar concepts:

1. **Callable** ≈ Function calls - "I want to do something now"
2. **Listener** ≈ Event handlers - "Tell me when something happens"
3. **Runnable** ≈ Background services - "Keep doing this until I say stop"

Teaching progression:
1. Start with simple Callable examples (function-like behavior)
2. Introduce Listeners with event handling concepts
3. Show Runnable for background service patterns
4. Demonstrate combining components for complex behaviors

Reference Implementation
========================

The reference implementation is available in the `owa-core` package:

- `owa.core.callable` - Callable interface and base implementations
- `owa.core.listener` - Listener interface with thread/process variants and callback management
- `owa.core.runnable` - Runnable interface with lifecycle management and context manager support

Example implementations demonstrating the design:
- Clock functions as Callables (time.time_ns)
- Keyboard/mouse event Listeners
- Background timer services as Runnables

Rejected Ideas
==============

### Synchronous-Only Design
Early consideration was given to maintaining compatibility with gymnasium-style synchronous interfaces. This was rejected because it would not address the core motivation of enabling real-time agent interactions.

### Complex Executor Hierarchy
The original executor design included separate TaskDefinition and ListenerDefinition classes with complex configuration patterns. This was simplified to the current unified Runnable interface for better usability.

### Centralized Event Bus
An alternative design considered a centralized event bus for all component communication. This was rejected in favor of direct callback registration for better performance and simpler debugging.

### Single Component Type
Early designs considered having only one component type that could handle all interaction patterns. This was rejected because it would create confusion about usage patterns and make the API less intuitive.

Open Issues
===========

At the time of this OEP's finalization, all major design decisions have been resolved and implemented. Future enhancements may be proposed in subsequent OEPs.

References
==========

- Original design discussions in `oep-0001/env.py` and `oep-0001/executor_design.py`
- Current implementation in `projects/owa-core/owa/core/`
- Component usage examples in `docs/env/guide.md`
- Registry and module system design in OEP-0002

Copyright
=========

This document is placed in the public domain or under the CC0-1.0-Universal license, whichever is more permissive.