OEP: 2
Title: Registry Pattern and Module System for OWA's Env
Author: Open World Agents Team
Status: Superseded
Type: Standards Track
Requires: 1
Created: 01-Feb-2025
Post-History: 07-Jun-2025
Superseded-By: 3

Abstract
========

This OEP describes the registry pattern and module system that enables dynamic component management and plugin activation in Open World Agents' Environment framework. Building upon the core components defined in OEP-1 (Callable, Listener, Runnable), this specification defines how components are registered, discovered, and activated through a centralized registry system and standardized plugin interface.

Motivation
==========

While OEP-1 established the fundamental component types (Callable, Listener, Runnable), a mechanism was needed to organize, discover, and activate these components dynamically. Traditional approaches require manual imports and explicit instantiation, which becomes unwieldy as the number of components grows and limits the framework's extensibility.

The registry pattern addresses several key requirements:

1. **Dynamic Component Discovery**: Enable runtime discovery of available components without requiring explicit imports
2. **Plugin Architecture**: Support for third-party plugins that can extend the framework without modifying core code
3. **Namespace Management**: Organize components in a hierarchical namespace to prevent naming conflicts
4. **Lazy Loading**: Load components only when needed to improve startup performance
5. **Standardized Activation**: Provide a consistent interface for activating environment modules

Without a registry system, users would need to manually import and manage every component, making the framework difficult to use and extend. The registry pattern provides a clean separation between component definition and component usage.

Rationale
=========

The registry design is based on several key principles:

**Centralized Management**: Three global registries (`CALLABLES`, `LISTENERS`, `RUNNABLES`) provide a single point of access for each component type, making discovery and usage straightforward.

**Decorator-Based Registration**: Components register themselves using decorators (`@CALLABLES.register("name")`), which is intuitive and keeps registration close to the component definition.

**Module-Based Activation**: The `activate_module()` function provides a standardized way to load and initialize environment plugins, following the `OwaEnvInterface` contract.

**Hierarchical Naming**: Component names use dot notation (e.g., "clock.time_ns") and slash notation (e.g., "clock/tick") to create logical groupings and prevent naming conflicts.

**Type Safety**: Generic registry types (`Registry[T]`) provide compile-time type checking while maintaining runtime flexibility.

The design balances simplicity with power, providing an easy-to-use interface while supporting complex plugin architectures.

Specification
=============

## Registry System

### Core Registry Classes

The registry system is built around a generic `Registry[T]` class:

```python
class Registry(Generic[T]):
    def register(self, name: str) -> Callable[[T], T]:
        """Decorator to register a component with the given name."""

    def __getitem__(self, name: str) -> T:
        """Retrieve a registered component by name."""

    def __contains__(self, name: str) -> bool:
        """Check if a component is registered."""

    def get(self, name: str) -> Optional[T]:
        """Safely retrieve a component, returning None if not found."""
```

### Global Registry Instances

Three global registries manage the core component types:

```python
CALLABLES: Registry[CallableCls] = Registry(registry_type=RegistryType.CALLABLES)
LISTENERS: Registry[ListenerCls] = Registry(registry_type=RegistryType.LISTENERS)
RUNNABLES: Registry[Runnable] = Registry(registry_type=RegistryType.RUNNABLES)
```

### Component Registration

Components register themselves using decorators:

```python
# Callable registration
@CALLABLES.register("clock.time_ns")
def time_ns_callable():
    return time.time_ns()

# Listener registration
@LISTENERS.register("clock/tick")
class ClockTickListener(Listener):
    def on_configure(self, *, interval=1):
        self.interval = interval

    def loop(self, *, stop_event, callback):
        # Implementation here
        pass

# Runnable registration
@RUNNABLES.register("example/counter")
class CounterRunnable(Runnable):
    def on_configure(self, *, start_value=0):
        self.value = start_value

    def loop(self, *, stop_event):
        # Implementation here
        pass
```

## Module System

### OwaEnvInterface

All environment plugins must implement the `OwaEnvInterface`:

```python
class OwaEnvInterface(ModuleType, ABC):
    @abstractmethod
    def activate(self):
        """
        Activate the environment plugin.

        This method should import modules that register callables, listeners,
        and runnables with the global registries.
        """
        ...
```

### Module Activation

The `activate_module()` function provides standardized plugin activation:

```python
def activate_module(entrypoint: str) -> OwaEnvInterface:
    """
    Activate a module by its entrypoint.

    Args:
        entrypoint: The module entrypoint to activate (e.g., "owa.env.desktop")

    Returns:
        The activated module instance

    Raises:
        Exception: If module not found or lacks activate() method
    """
```

### Plugin Structure

Environment plugins follow a standard structure:

```python
# In plugin's __init__.py
def activate():
    """Activate the plugin by importing modules that register components."""
    from . import my_callables  # noqa
    from . import my_listeners  # noqa
    from . import my_runnables  # noqa
```

## Naming Conventions

### Component Naming

- **Callables**: Use dot notation for hierarchical organization (e.g., "clock.time_ns", "mouse.click", "window.get_active")
- **Listeners**: Use slash notation to indicate event streams (e.g., "clock/tick", "keyboard", "mouse/move")
- **Runnables**: Use slash notation for service-like components (e.g., "example/counter", "data/recorder")

### Module Naming

- Environment modules use the pattern: `owa.env.{domain}` (e.g., "owa.env.std", "owa.env.desktop")
- Third-party plugins can use their own namespace but should follow similar patterns

## Usage Patterns

### Basic Usage

```python
from owa.core.registry import CALLABLES, LISTENERS, activate_module

# Activate a module
activate_module("owa.env.std")

# Use registered components
current_time = CALLABLES["clock.time_ns"]()
tick_listener = LISTENERS["clock/tick"]().configure(callback=my_callback, interval=1)
```

### Custom Plugin Development

```python
# Define components with registration
@CALLABLES.register("my_plugin.add")
def add_numbers(a: int, b: int) -> int:
    return a + b

@LISTENERS.register("my_plugin/events")
class EventListener(Listener):
    # Implementation here
    pass
```

Backwards Compatibility
=======================

This OEP introduces the registry system as a new feature and does not break existing functionality. However, it establishes patterns that future components should follow:

1. **Component Registration**: All new components should use the registry decorators rather than manual registration
2. **Module Structure**: New environment plugins should implement `OwaEnvInterface` and follow the standard activation pattern
3. **Naming Conventions**: Component names should follow the established dot/slash notation patterns

Existing code that directly imports and uses components will continue to work, but using the registry system is recommended for better discoverability and consistency.

Security Implications
=====================

The registry and module system introduces several security considerations:

1. **Code Execution**: The `activate_module()` function imports and executes arbitrary Python modules, which could contain malicious code
2. **Namespace Pollution**: Malicious plugins could register components with names that override legitimate components
3. **Resource Consumption**: Poorly implemented plugins could consume excessive system resources during activation

Mitigation strategies include:

- **Trusted Sources**: Only activate modules from trusted sources
- **Sandboxing**: Consider running untrusted plugins in isolated environments
- **Validation**: Validate component names and implementations before registration
- **Resource Limits**: Implement timeouts and resource limits for plugin activation

How to Teach This
=================

The registry system can be taught through a progressive approach:

1. **Basic Usage**: Start with using pre-registered components from standard modules
   ```python
   activate_module("owa.env.std")
   time_ns = CALLABLES["clock.time_ns"]()
   ```

2. **Component Discovery**: Show how to explore available components
   ```python
   print(CALLABLES)  # See all registered callables
   print("clock.time_ns" in CALLABLES)  # Check if component exists
   ```

3. **Custom Registration**: Demonstrate registering custom components
   ```python
   @CALLABLES.register("my.function")
   def my_function():
       return "Hello, World!"
   ```

4. **Plugin Development**: Guide through creating complete environment plugins with proper structure and activation patterns

The key concepts to emphasize:
- **Separation of Concerns**: Registration happens once, usage happens many times
- **Discoverability**: The registry makes components easy to find and use
- **Extensibility**: Anyone can add new components without modifying core code

Reference Implementation
========================

The reference implementation is available in `projects/owa-core/owa/core/registry.py` and includes:

- **Registry Class**: Generic registry with type safety and standard dictionary-like interface
- **Global Registries**: Pre-configured instances for CALLABLES, LISTENERS, and RUNNABLES
- **Module Activation**: `activate_module()` function with error handling and validation
- **Interface Definition**: `OwaEnvInterface` abstract base class for plugin contracts

Example implementations demonstrating the system:

- **Standard Module** (`owa.env.std`): Clock functions and tick listeners
- **Desktop Module** (`owa.env.desktop`): Screen capture, window management, input handling
- **Example Plugin** (`owa.env.example`): Simple demonstration plugin showing all component types

The implementation has been tested with multiple environment plugins and demonstrates the scalability and flexibility of the registry approach.

Rejected Ideas
==============

### Centralized Configuration Files

An alternative approach considered using configuration files (JSON/YAML) to define component mappings. This was rejected because:

- It separates component definition from registration, making maintenance harder
- It requires additional parsing and validation logic
- It doesn't provide the same level of type safety as decorator-based registration

### Automatic Discovery via Filesystem Scanning

Automatic discovery of components by scanning the filesystem was considered but rejected due to:

- Security concerns about executing arbitrary code
- Performance implications of filesystem scanning
- Difficulty in determining component interfaces without execution
- Complexity in handling different package structures

### Single Global Registry

Using a single registry for all component types was considered but rejected because:

- It would lose type safety benefits
- Component names would need prefixes to avoid conflicts
- It would make the API less intuitive for users
- Different component types have different usage patterns that benefit from separation

### Plugin Marketplace Integration

Integration with a centralized plugin marketplace was discussed but deemed outside the scope of this OEP. Such functionality could be added in future OEPs without changing the core registry design.

Open Issues
===========

At the time of this OEP's finalization, the registry system is fully implemented and in active use. Future enhancements that may be considered include:

1. **Plugin Versioning**: Support for version constraints and compatibility checking
2. **Lazy Loading Optimization**: More sophisticated lazy loading strategies for large plugin ecosystems
3. **Registry Introspection**: Enhanced tools for exploring and documenting registered components
4. **Plugin Dependencies**: Formal dependency management between plugins

These enhancements would be addressed in future OEPs as the ecosystem grows and requirements become clearer.

References
==========

- OEP-1: Core Component Design of OWA's Env - Callable, Listener, and Runnable
- Implementation: `projects/owa-core/owa/core/registry.py`
- Interface Definition: `projects/owa-core/owa/core/owa_env_interface.py`
- Usage Examples: `docs/env/guide.md`
- Standard Plugin: `projects/owa-core/owa/env/std/`
- Example Plugin: `projects/owa-env-example/owa/env/example/`
- MMEngine Registry Documentation: https://mmengine.readthedocs.io/en/latest/advanced_tutorials/registry.html

Copyright
=========

This document is placed in the public domain or under the CC0-1.0-Universal license, whichever is more permissive.
