OEP: 3
Title: Entry Points-Based Plugin Discovery and Unified Component Naming
Author: Open World Agents Team
Status: Final
Type: Standards Track
Requires: 1, 2
Created: 07-Jun-2025
Post-History: 07-Jun-2025
Replaces: 2

Abstract
========

This OEP introduces a standardized plugin discovery mechanism using Python's Entry Points system and establishes unified naming conventions for all OWA components. Building upon the registry system defined in OEP-2, this specification eliminates the need for manual `activate_module()` calls by implementing automatic plugin discovery, introduces lazy loading for optimal performance, and unifies component naming to use a consistent `namespace/name` pattern across all component types.

Motivation
==========

The registry system established in OEP-2 provided a foundation for component management but had several limitations that hindered developer experience and system scalability:

1. **Manual Activation Required**: Developers had to explicitly call `activate_module()` for each plugin, creating friction and potential for errors
2. **Inconsistent Naming**: Callables used dot notation (`clock.time_ns`) while Listeners and Runnables used slash notation (`clock/tick`), creating confusion
3. **Non-Standard Discovery**: Custom module activation didn't follow Python packaging standards, limiting ecosystem integration
4. **Performance Issues**: All components were loaded at activation time, causing slow startup with many plugins
5. **Limited Discoverability**: No standard way for users to discover available plugins without manual documentation

These limitations became apparent as the OWA ecosystem grew and more third-party plugins were developed. A more robust, standards-compliant approach was needed to support a thriving plugin ecosystem.

Rationale
=========

The design is based on Python's official packaging standards and modern plugin architecture best practices:

**Python Standards Compliance**: Entry Points are the official Python mechanism for plugin discovery, as documented in the Python Packaging Guide. This ensures compatibility with standard Python tooling and packaging workflows.

**Zero-Configuration Usage**: Plugins work immediately after installation via `pip install`, eliminating the need for manual activation calls and reducing cognitive load for users.

**Unified Naming Convention**: All components now use the `namespace/name` pattern, providing consistency and eliminating the confusion between different notation systems.

**Lazy Loading Architecture**: Components are imported only when accessed, not when discovered, providing fast startup times and minimal memory usage even with many installed plugins.

**Enhanced Developer Experience**: Multiple access patterns (`CALLABLES["namespace/name"]` and `get_component("callables", "namespace", "name")`) provide flexibility while maintaining backward compatibility.

The design prioritizes developer experience, performance, and standards compliance while maintaining the flexibility and power of the existing registry system.

Specification
=============

## Entry Points-Based Plugin Declaration

### Plugin Package Structure

Plugins declare themselves using standard Python Entry Points in `pyproject.toml`:

```toml
[project]
name = "owa-env-example"
version = "0.1.0"
description = "Example environment plugin for Open World Agents"
dependencies = ["owa-core"]

[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"

[project.entry-points."owa.env.plugins"]
example = "owa.env.plugins.example:plugin_spec"
```

### Plugin Specification

Each plugin must provide a `PluginSpec` object in a separate module to avoid circular imports during plugin discovery. The specification should be placed in `owa/env/plugins/<namespace>.py`:

```python
# File: owa/env/plugins/example.py
"""
Plugin specification for the Example environment plugin.

This module is kept separate to avoid circular imports during plugin discovery.
"""

from owa.core.plugin_spec import PluginSpec

plugin_spec = PluginSpec(
    namespace="example",
    version="0.1.0",
    description="Example environment plugin",
    author="Plugin Developer",
    components={
        "callables": {
            "add": "owa.env.example:add_numbers",
            "print": "owa.env.example:enhanced_print",
        },
        "listeners": {
            "events": "owa.env.example:EventListener",
        },
        "runnables": {
            "processor": "owa.env.example:DataProcessor",
        }
    }
)
```

**Important**: Do not create `__init__.py` files in the `plugins/` directory as this would interfere with namespace packaging.

### Plugin Module Structure Flexibility

**Zero Structure Limitations**: Custom environment plugins have complete freedom in their module organization. As long as the entry point is properly registered, the internal structure of your plugin module can be organized however you prefer.

**Required Structure**:
```
owa/env/plugins/
└── myplugin.py              # Plugin specification (required)
owa/env/myplugin/
└── __init__.py              # Your main module (required)
└── [your code structure]    # Organize however you want!
```

**Example Valid Structures**:
```
# Minimal structure
owa/env/myplugin/
├── __init__.py
└── components.py

# Organized by component type
owa/env/myplugin/
├── __init__.py
├── callables/
│   ├── math.py
│   └── utils.py
├── listeners/
│   └── events.py
└── runnables/
    └── processors.py

# Domain-driven structure
owa/env/myplugin/
├── __init__.py
├── core/
│   ├── engine.py
│   └── config.py
├── integrations/
│   ├── api.py
│   └── database.py
└── utils/
    └── helpers.py

# Flat structure
owa/env/myplugin/
├── __init__.py
├── feature_a.py
├── feature_b.py
└── shared_utils.py
```

The entry point system provides complete flexibility - organize your code in whatever way makes sense for your plugin's functionality and complexity.

## Unified Component Naming

### Naming Convention

All components now use the unified `namespace/name` pattern:

- **Callables**: `example/add`, `std/time_ns`, `desktop/click`
- **Listeners**: `example/events`, `std/tick`, `keyboard/press`  
- **Runnables**: `example/processor`, `std/monitor`, `desktop/recorder`

### Naming Rules

- **Namespace**: Must consist of letters, numbers, underscores, and hyphens: `^[a-zA-Z0-9_-]+$`
- **Name**: Should consist of letters, numbers, underscores, and dots: `^[a-zA-Z_][a-zA-Z0-9_.]*$`
- **Separator**: Always use forward slash (`/`) between namespace and name
- **Functional Grouping**: Use dots within names for logical grouping (e.g., `mouse.click`, `keyboard.type`)

## Automatic Plugin Discovery

### Discovery Process

The system automatically discovers plugins when `owa.core` is imported:

1. **Entry Point Scanning**: System scans all installed packages for entry points in the `"owa.env.plugins"` group
2. **Plugin Specification Loading**: For each entry point, load the specified `PluginSpec` object
3. **Component Registration**: Register component metadata (import paths) with appropriate registries
4. **Lazy Loading Setup**: Components remain unloaded until first access

### Zero-Configuration Usage

Users install plugins using standard pip and components are immediately available:

```bash
pip install owa-env-example
```

```python
from owa.core.registry import CALLABLES, LISTENERS, RUNNABLES

# Components automatically available - no activate_module() needed
result = CALLABLES["example/add"](5, 3)
listener = LISTENERS["example/events"]().configure(callback=my_callback)
processor = RUNNABLES["example/processor"]().configure(batch_size=100)
```

## Lazy Loading Architecture

### LazyImportRegistry

The `LazyImportRegistry` extends the base `Registry` class with lazy loading capabilities. Components are registered with import paths rather than loaded objects, enabling fast startup times and minimal memory usage.

### Loading Process

1. **Registration Phase**: Only metadata and import paths stored, no imports occur
2. **Access Phase**: Component imported when retrieved from registry (`CALLABLES["name"]`)
3. **Caching Phase**: Imported components cached for subsequent access
4. **Error Handling**: Import errors occur at access time, allowing graceful degradation

See `projects/owa-core/owa/core/registry.py` for the complete implementation.

## Enhanced Component Access API

### Multiple Access Patterns

The system provides flexible component access methods:

```python
from owa.core import CALLABLES, get_component, list_components

# Pattern 1: Direct registry access
add_func = CALLABLES["example/add"]
result = add_func(5, 3)

# Pattern 2: Function-based access with parameters
add_func = get_component("callables", namespace="example", name="add")
result = add_func(5, 3)

# Pattern 3: Namespace-based access
example_callables = get_component("callables", namespace="example")
# Returns: {"add": <function>, "print": <function>}

# Pattern 4: Component discovery
all_callables = list_components("callables")
example_only = list_components("callables", namespace="example")
```

### API Functions

The system provides several utility functions for component access and discovery:
- `get_component()` - Flexible component access with multiple patterns
- `list_components()` - List available components with optional filtering
- `get_registry()` - Get the appropriate registry for component type

See `projects/owa-core/owa/core/api.py` for complete function signatures and implementation.

Backwards Compatibility
=======================

This OEP introduces breaking changes to the component naming system and plugin activation mechanism:

**Breaking Changes:**
1. **Naming Convention**: Callables must migrate from dot notation (`clock.time_ns`) to slash notation (`std/time_ns`)
2. **Activation Method**: `activate_module()` calls are no longer needed and should be removed
3. **Registration Method**: Decorator-based registration replaced with specification-based registration

**Migration Path:**

*Plugin Developers:*
- Add entry point declaration to `pyproject.toml`
- Create `PluginSpec` in separate `owa/env/plugins/<namespace>.py` file
- Update component names to unified `namespace/name` pattern
- Replace decorator registration with specification-based registration

*Plugin Users:*
- Remove `activate_module()` calls from user code
- Update component names from dot notation to slash notation for callables
- Ensure plugins are installed via pip rather than manually imported

**Migration Support:**
- Comprehensive migration guide provided in documentation
- Example implementations demonstrate new patterns
- Gradual migration path allows incremental adoption

**Compatibility Timeline:**
- OEP-2 style plugins continue to work during transition period
- Full migration required for optimal performance and feature access
- Legacy support may be removed in future major versions

Security Implications
=====================

The Entry Points system introduces new security considerations:

**Reduced Attack Surface:**
- Eliminates arbitrary module execution via `activate_module()`
- Components loaded only when accessed, limiting exposure
- Specification-based registration provides better validation opportunities

**New Considerations:**
- Entry Points can still reference malicious code
- Lazy loading defers but doesn't eliminate code execution risks
- Plugin specifications should be validated before registration

**Mitigation Strategies:**
- Validate plugin specifications during discovery
- Implement component access logging for security monitoring
- Consider sandboxing for untrusted plugins
- Provide tools for plugin verification and auditing

How to Teach This
=================

The Entry Points system can be taught through progressive examples:

### 1. Basic Usage (Consumer Perspective)
```python
# Install and use - no configuration needed
pip install owa-env-example
from owa.core import CALLABLES
result = CALLABLES["example/add"](5, 3)
```

### 2. Component Discovery
```python
from owa.core import list_components, get_component
print(list_components("callables"))  # See what's available
add_func = get_component("callables", "example", "add")
```

### 3. Plugin Development
```python
# File: owa/env/plugins/myplugin.py
"""Plugin specification for MyPlugin environment plugin."""

from owa.core.plugin_spec import PluginSpec

plugin_spec = PluginSpec(
    namespace="myplugin",
    components={"callables": {"hello": "owa.env.myplugin:say_hello"}}
)
```

```toml
# pyproject.toml entry point
[project.entry-points."owa.env.plugins"]
myplugin = "owa.env.plugins.myplugin:plugin_spec"
```

### 4. Advanced Patterns
- Multiple access methods
- Lazy loading behavior
- Error handling strategies
- Performance considerations

**Key Teaching Points:**
- **Standards Compliance**: Emphasize that this follows Python packaging standards
- **Zero Configuration**: Highlight the elimination of manual activation
- **Performance Benefits**: Explain lazy loading advantages
- **Unified Experience**: Show consistency across component types

## Enhanced CLI Tools

The `owl env` command-line interface provides comprehensive plugin and component management capabilities:

### Core Commands

- `owl env list` - List all discovered plugins with filtering options
- `owl env show <namespace>` - Show detailed plugin information
- `owl env search <pattern>` - Search for components across plugins
- `owl env stats` - Display ecosystem statistics and health information
- `owl env validate` - Validate plugin specifications

### Key Features

- **Automatic Discovery**: No manual activation required - plugins discovered via entry points
- **Rich Filtering**: Filter by namespace, component type, or search patterns
- **Multiple Output Formats**: Table, tree, and detailed views available
- **Performance Monitoring**: Shows lazy loading status and component statistics
- **Development Support**: Validation tools for plugin developers

See `projects/owa-cli/owa/cli/env/` for complete CLI implementation and available options.

Reference Implementation
========================

The reference implementation is complete and available in the Open World Agents repository:

**Core Components:**
- `owa.core.registry.LazyImportRegistry` - Lazy loading registry implementation
- `owa.core.plugin_spec.PluginSpec` - Plugin specification with validation
- `owa.core.discovery` - Entry points-based plugin discovery
- `owa.core.api` - Enhanced component access functions

**Example Implementations:**
- `projects/owa-env-example/` - Complete example plugin demonstrating all patterns
- `projects/owa-env-desktop/` - Desktop environment plugin with comprehensive component set
- `projects/owa-env-gst/` - GStreamer plugin demonstrating multimedia capabilities
- `projects/owa-core/owa/env/std/` - Standard library components

**Validation:**
The implementation has been thoroughly tested and validated with real-world plugins, demonstrating significant improvements in startup performance and developer experience. See the respective project directories for detailed implementation information.

Rejected Ideas
==============

### Configuration File-Based Discovery

Using configuration files (JSON/YAML) for plugin discovery was considered but rejected because:
- Entry Points are the Python standard for plugin discovery
- Configuration files require additional parsing and validation
- Entry Points integrate better with packaging tools
- Metadata is already available through package management

### Backward Compatible Naming

Maintaining both dot and slash notation was considered but rejected because:
- It would perpetuate confusion between component types
- Implementation complexity would increase significantly
- Clear migration path is preferable to indefinite dual support
- Unified naming provides better long-term developer experience

### Immediate Loading Option

Providing an option to disable lazy loading was considered but rejected because:
- Lazy loading provides significant performance benefits
- Edge cases requiring immediate loading can be handled through eager_load parameter
- Complexity of dual loading modes outweighs benefits
- Lazy loading is becoming standard practice in modern frameworks

### Centralized Plugin Registry

A centralized online registry for plugins was discussed but deemed outside scope because:
- Entry Points already provide decentralized discovery
- Centralized registries introduce single points of failure
- Package managers (PyPI) already serve this function
- Focus should remain on technical implementation rather than ecosystem management

Open Issues
===========

At the time of this OEP's finalization, the core system is implemented and functional. Future enhancements may include:

1. **Enhanced Type Safety**: Better IDE support and type checking for component access
2. **Plugin Validation Tools**: Automated validation and testing tools for plugin developers
3. **Performance Monitoring**: Tools for monitoring lazy loading performance and optimization
4. **Migration Tooling**: Automated tools for migrating OEP-2 style plugins

These enhancements would be addressed in future OEPs as the ecosystem matures and requirements become clearer.

References
==========

- OEP-1: Core Component Design of OWA's Env - Callable, Listener, and Runnable
- OEP-2: Registry Pattern and Module System for OWA's Env
- Python Packaging Guide: https://packaging.python.org/en/latest/guides/creating-and-discovering-plugins/
- Entry Points Specification: https://packaging.python.org/en/latest/specifications/entry-points/
- Implementation: `projects/owa-core/owa/core/`
- Example Plugin: `projects/owa-env-example/`
- Migration Guide: `docs/migration/oep-3.md`

Copyright
=========

This document is placed in the public domain or under the CC0-1.0-Universal license, whichever is more permissive.
