OEP: 4
Title: EnvPlugin Documentation Validation and Custom mkdocstrings Handler
Author: Open World Agents Team
Status: Final
Type: Standards Track
Requires: 3
Created: 08-Jun-2025
Post-History: 08-Jun-2025

Abstract
========

This OEP introduces a documentation validation system for EnvPlugins and a custom mkdocstrings handler for automatic documentation generation. The primary focus is on providing a `owl env validate-docs` command that can be used as a test utility for plugin quality assurance, with proper exit codes for CI/CD integration. Additionally, a minimal custom mkdocstrings handler enables automatic documentation generation from existing docstrings without manual maintenance.

Motivation
==========

The current OEP-3 plugin system lacks systematic documentation validation and automated documentation generation:

1. **No Documentation Quality Assurance**: There's no standardized way to validate that plugins have adequate documentation, making it difficult to maintain quality standards across the ecosystem
2. **Missing CI/CD Integration**: Plugin contributors need a test command that can validate documentation quality with proper exit codes for automated testing
3. **Manual Documentation Maintenance**: Plugin developers must manually maintain separate documentation files, leading to synchronization issues
4. **Limited mkdocstrings Integration**: While mkdocstrings works well for regular Python packages, it doesn't understand OWA's plugin structure and component organization

These limitations create barriers for both plugin quality assurance and documentation automation in CI/CD pipelines.

Rationale
=========

The design focuses on two key areas: documentation validation for quality assurance and minimal mkdocstrings integration:

**Documentation Validation as Testing**: The `owl env validate-docs` command serves as a test utility that can be integrated into CI/CD pipelines, ensuring consistent documentation quality across all plugins with proper exit codes.

**Minimal mkdocstrings Handler**: A lightweight custom handler that understands OWA's plugin structure, enabling automatic documentation generation without complex infrastructure.

**CI/CD Integration**: Proper stdout/stderr output and exit codes make documentation validation part of the standard testing workflow for plugin contributions.

**Zero-Configuration Approach**: Both validation and documentation generation work out-of-the-box with existing plugin structures, requiring no additional configuration files or metadata.

Specification
=============

## Documentation Validation Command

### `owl env validate-docs` as Test Utility

The primary feature is a validation command that can be used as a test in CI/CD pipelines:

```bash
# Basic validation with quality grades
$ owl env validate-docs
✅ std plugin: 2/2 documented (100%), 2/2 good quality (100%)
⚠️  example plugin: 7/7 documented (100%), 3/7 good quality (43%)
❌ custom plugin: 1/5 documented (20%), 0/5 good quality (0%)

# Validation for specific plugin
$ owl env validate-docs example

# Strict mode for CI/CD (enforces 100% coverage + 100% quality)
$ owl env validate-docs --strict

# JSON output for tooling integration
$ owl env validate-docs --format json
```

### Exit Code Specification

The command follows standard Unix conventions for test utilities:

- **Exit Code 0**: All validations passed
- **Exit Code 1**: Documentation issues found (warnings or failures)
- **Exit Code 2**: Command error (invalid arguments, plugin not found, etc.)

### Validation Criteria

#### Component-Level Quality Grades

**GOOD** ✅ (High quality documentation)
- Has non-empty docstring
- Has examples AND type hints AND comprehensive description (>50 characters)
- Meets basic quality standards

**ACCEPTABLE** 📝 (Basic documentation)
- Has non-empty docstring
- Missing examples, type hints, or comprehensive description
- Still counts as "documented" but lower quality

**POOR** ❌ (No documentation)
- No docstring or empty docstring
- Component is considered "undocumented"

#### Per-Plugin Quality Thresholds

**PASS** ✅ (Plugin is well-documented)
- ≥ 80% of components are GOOD or ACCEPTABLE
- ≥ 60% of components are GOOD (high quality)

**WARN** ⚠️ (Plugin needs improvement)
- 60-79% of components are documented (GOOD or ACCEPTABLE)
- OR < 60% of components are GOOD quality

**FAIL** ❌ (Plugin is poorly documented)
- < 60% of components are documented

#### Skip Quality Check Feature

Components can opt out of quality checks using docstring comments:

```python
def legacy_function():
    """Legacy function with minimal docs.

    @skip-quality-check: legacy-code
    """
    pass

class InternalClass:
    """Internal implementation class.

    @skip-quality-check: internal-api
    """
    pass
```

**Valid skip reasons**: `legacy-code`, `internal-api`, `experimental`, `deprecated`, `third-party`

**Note**: Skipped components are excluded from quality calculations entirely.

**Note**: Detailed validation logic is implemented in the codebase at `projects/owa-core/owa/core/documentation/validator.py`.

## Custom mkdocstrings Handler

### Minimal Handler Implementation

A lightweight custom mkdocstrings handler that understands OWA plugin structure:

**Implementation Note**: The handler includes graceful degradation when mkdocstrings is not available, ensuring the core functionality remains intact even in minimal environments.

**Note**: The mkdocstrings handler is implemented as a separate package `mkdocstrings-owa` available on PyPI and developed at https://github.com/open-world-agents/mkdocstrings-owa.

### Usage in Documentation

The handler enables automatic documentation generation using familiar mkdocstrings syntax:

```markdown
# Plugin overview
::: example
    handler: owa

# Individual component
::: example/mouse.click
    handler: owa
    options:
      show_signature: true
      show_examples: true
```

### Integration Setup

**Note**: The OWA mkdocstrings handler is distributed as the `mkdocstrings-owa` package on PyPI (source: https://github.com/open-world-agents/mkdocstrings-owa) and is automatically available when installed.

```yaml
# mkdocs.yml - Configure the handler
plugins:
  - mkdocstrings:
      handlers:
        owa:
          options:
            show_plugin_metadata: true
            include_source_links: true
```

**Note**: The OWA handler is distributed as the `mkdocstrings-owa` package (https://github.com/open-world-agents/mkdocstrings-owa) and is automatically available when installed via pip. It integrates seamlessly with the existing mkdocstrings workflow.

## CI/CD Integration

### Using as Test Command

The validation command integrates seamlessly into testing workflows:

- **CI/CD Pipelines**: Use `owl env validate-docs --strict` in GitHub Actions, GitLab CI, etc.
- **Pre-commit Hooks**: Validate documentation before commits
- **pytest Integration**: Include documentation validation in test suites
- **JSON Output**: Machine-readable format for tooling integration

**Note**: A working GitHub Actions workflow is implemented in `.github/workflows/validate-docs.yml` that demonstrates the CI/CD integration.

### Repository-wide Application

The command supports validation across entire repositories with configurable thresholds and output formats for comprehensive quality assurance.

Reference Implementation
========================

The reference implementation is complete and available in the Open World Agents repository:

**Core Components:**
- `projects/owa-core/owa/core/documentation/validator.py` - Documentation validation engine
- `mkdocstrings-owa` (external package at https://github.com/open-world-agents/mkdocstrings-owa) - Custom mkdocstrings handler for OWA plugins
- `projects/owa-cli/owa/cli/env/docs.py` - CLI commands with validation support

**Key Features:**
- Exit code compliance for CI/CD integration (0=pass, 1=fail, 2=error)
- JSON output format for tooling integration
- Configurable coverage thresholds and quality checks
- Repository-wide validation support
- Lightweight custom handler for OWA plugin structure
- Zero-configuration setup for existing plugins

**Note**: The mkdocstrings handler is implemented as the separate `mkdocstrings-owa` package available on PyPI (source: https://github.com/open-world-agents/mkdocstrings-owa).

Backwards Compatibility
=======================

This OEP introduces new functionality without breaking existing systems:

**No Breaking Changes:**
- All existing plugins continue to work without modification
- Documentation validation is an optional tool for quality assurance
- mkdocstrings integration is opt-in and doesn't affect existing documentation workflows

**Additive Features:**
- New `owl env validate-docs` command for documentation quality assurance
- Optional mkdocstrings handler for automatic documentation generation
- CI/CD integration capabilities for documentation validation

**Migration Path:**
- Existing plugins can gradually improve documentation quality
- No immediate action required for existing codebases
- Documentation validation can be adopted incrementally in CI/CD pipelines

Security Implications
=====================

The documentation validation system introduces minimal security considerations:

**Validation Security:**
- Documentation validation only reads existing code and docstrings
- No code execution or modification during validation process
- JSON output format prevents injection attacks in tooling integration

**mkdocstrings Integration:**
- Uses standard mkdocstrings security model
- No additional security risks beyond existing mkdocstrings usage
- Handler follows Python packaging security best practices

**CI/CD Integration:**
- Standard exit codes and output formats prevent security issues
- No sensitive information exposed in validation reports
- Compatible with existing CI/CD security practices

How to Teach This
=================

The documentation validation system can be taught through progressive adoption:

### 1. Basic Usage
```bash
# Start with simple validation
owl env validate-docs
```

### 2. CI/CD Integration
- Add validation to existing test suites
- Use proper exit codes for automated testing
- Integrate with pre-commit hooks for quality gates

### 3. Quality Improvement
- Understand quality grades (GOOD, ACCEPTABLE, POOR)
- Use configurable thresholds for gradual improvement
- Apply skip-quality-check annotations for special cases

### 4. Documentation Generation
- Install mkdocstrings-owa package
- Use familiar mkdocstrings syntax for OWA plugins
- Integrate with existing documentation workflows

**Key Teaching Points:**
- **Quality Assurance Focus**: Emphasize documentation as part of code quality
- **CI/CD Integration**: Show how validation fits into existing testing workflows
- **Gradual Adoption**: Demonstrate incremental improvement strategies
- **Zero Configuration**: Highlight out-of-the-box functionality

Rejected Ideas
==============

### Complex Documentation Generation System

A comprehensive documentation generation system similar to Sphinx was considered but rejected because:
- Adds significant complexity and maintenance overhead
- Conflicts with the minimal, performant approach requested
- mkdocstrings already provides excellent documentation generation
- Focus should be on validation and minimal integration, not reinventing documentation tools

### Runtime Documentation Validation

Validating documentation at plugin load time was considered but rejected because:
- Would impact the performance benefits of OEP-3's lazy loading
- Unnecessary overhead for users who don't need documentation validation
- Conflicts with the on-demand principle of the plugin system
- Documentation validation should be a development/CI tool, not a runtime feature

### Comprehensive Template System

A full template system with extensive customization options was considered but rejected because:
- Adds complexity without clear benefit for the minimal use case
- mkdocstrings already provides excellent templating capabilities
- Focus should be on integration, not reimplementing existing functionality
- Simple templates are sufficient for the basic handler needs

Open Issues
===========

While the core specification is complete, several areas warrant further discussion:

### Documentation Quality Standards

- Should there be minimum documentation requirements for plugins?
- How should documentation quality be measured and enforced?
- What guidelines should be provided for effective plugin documentation?

### Integration with Package Managers

- Should documentation validation be integrated with PyPI package publishing?
- How should documentation versioning align with package versioning?
- What opportunities exist for documentation-based package discovery?

References
==========

- **OEP-3**: Entry Points-Based Plugin Discovery and Unified Component Naming
- **mkdocstrings-owa Handler**: https://github.com/open-world-agents/mkdocstrings-owa
- **mkdocstrings Documentation**: https://mkdocstrings.github.io/
- **Python Packaging Guide**: https://packaging.python.org/en/latest/
- **Google Style Docstrings**: https://google.github.io/styleguide/pyguide.html

Copyright
=========

This document is placed in the public domain or under the CC0-1.0-Universal license, whichever is more permissive.
