OEP: 5
Title: Message Definition Registration in EnvPlugin System
Author: Open World Agents Team
Status: Rejected
Type: Standards Track
Requires: 3
Created: 10-Jun-2025
Post-History: 10-Jun-2025
Superseded-By: 6

Abstract
========

This OEP extends the EnvPlugin system established in OEP-3 to support registration of custom message definitions alongside Callables, Listeners, and Runnables. This enhancement enables plugins to define their own OWAMessage types for use in OWAMcap files, creating a unified plugin ecosystem where both runtime components and data schemas are managed together. The proposal includes updates to the PluginSpec format, mcap-owa-support package integration, and enhanced CLI tooling for message discovery and validation.

Motivation
==========

The current EnvPlugin system allows registration of runtime components (Callables, Listeners, Runnables) but lacks support for custom message definitions. This creates several limitations:

1. **Fragmented Plugin Development**: Plugin developers must manage message definitions separately from their runtime components, leading to inconsistent versioning and deployment
2. **Limited OWAMcap Extensibility**: Custom message types require manual registration with mcap-owa-support, preventing dynamic plugin-based message discovery
3. **Incomplete Plugin Ecosystem**: Plugins cannot fully encapsulate their functionality since they cannot define the data structures they work with
4. **Discovery Gaps**: No standardized way to discover what message types are available from installed plugins
5. **Integration Complexity**: Separate management of messages and components complicates plugin installation and usage

These limitations become apparent when developing domain-specific plugins that need custom data structures for specialized desktop interactions, sensor data, or application-specific events.

Rationale
=========

The design extends the existing Entry Points-based plugin system with message definition support:

**Unified Plugin Management**: Message definitions become first-class citizens in the plugin system, managed alongside runtime components with consistent versioning and lifecycle management.

**Dynamic Message Discovery**: The mcap-owa-support package can dynamically discover and register message types from installed plugins, eliminating manual registration steps.

**Enhanced Developer Experience**: Plugin developers define everything in one PluginSpec, ensuring consistency between runtime components and the data structures they use.

**Standards Compliance**: Builds upon the existing Entry Points system without introducing new discovery mechanisms, maintaining consistency with OEP-3.

**Backward Compatibility**: Existing plugins continue to work unchanged, with message registration being an optional enhancement.

Specification
=============

## Extended PluginSpec Format

### Message Definition Registration

Plugins declare message definitions in their PluginSpec alongside existing components:

```python
from owa.core.plugin_spec import PluginSpec
from owa.core.message import OWAMessage

class CustomSensorData(OWAMessage):
    _type = "custom_plugin/SensorData"
    timestamp: int
    sensor_id: str
    values: list[float]
    metadata: dict

plugin_spec = PluginSpec(
    namespace="custom_plugin",
    version="0.1.0",
    description="Custom sensor plugin with message definitions",
    author="Plugin Developer",
    components={
        "callables": {
            "process_sensor": "custom_plugin:process_sensor_data",
        },
        "listeners": {
            "sensor_events": "custom_plugin:SensorEventListener",
        },
        "runnables": {
            "data_collector": "custom_plugin:SensorDataCollector",
        }
    },
    messages={
        "SensorData": "custom_plugin:CustomSensorData",
        "SensorEvent": "custom_plugin:CustomSensorEvent",
        "ProcessingResult": "custom_plugin:ProcessingResult",
    }
)
```

## Message Definition Requirements

### OWAMessage Base Class

All registered message definitions must inherit from `OWAMessage`:

```python
from owa.core.message import OWAMessage

class CustomMessage(OWAMessage):
    _type = "namespace/MessageType"  # Required: unique message type identifier
    field1: str                      # Type hints required for schema generation
    field2: int
    optional_field: str = None       # Optional fields supported
```

### Message Type Naming Convention

Message types follow the same namespace/name pattern as components:
- **Format**: `namespace/MessageType`
- **Namespace**: Must match the plugin's namespace
- **MessageType**: PascalCase naming convention
- **Examples**: `desktop/KeyboardEvent`, `sensor/TemperatureReading`, `custom/ProcessingResult`

## Integration with mcap-owa-support

### Enhanced OWAMcapWriter/Reader

Message types are automatically available in OWAMcap operations:

```python
from mcap_owa.highlevel import OWAMcapWriter
from custom_plugin import CustomSensorData

with OWAMcapWriter("sensor_data.mcap") as writer:
    # Custom message types automatically available
    sensor_msg = CustomSensorData(
        timestamp=1234567890,
        sensor_id="temp_01",
        values=[23.5, 24.1, 23.8],
        metadata={"location": "room_a"}
    )
    writer.write_message("sensors", sensor_msg)
```

## Enhanced CLI Support

### Message Discovery Commands

```bash
# List all available message types
$ owl env messages

# Show messages from specific namespace
$ owl env messages --namespace custom_plugin

# Show detailed message schema
$ owl env show-message custom_plugin/SensorData

# Validate message definitions
$ owl env validate-messages custom_plugin
```

### Integration with Existing Commands

```bash
# Show plugin with message information
$ owl env show custom_plugin --messages

# List components and messages together
$ owl env list --include-messages
```

Evolution from OEP-3
====================

## Key Additions to Plugin System

### Before (OEP-3 System)
```python
plugin_spec = PluginSpec(
    namespace="example",
    components={
        "callables": {"process": "example:process_data"},
    }
)
# Message definitions managed separately
```

### After (OEP-5 System)
```python
plugin_spec = PluginSpec(
    namespace="example",
    components={
        "callables": {"process": "example:process_data"},
    },
    messages={
        "InputData": "example:InputDataMessage",
        "OutputData": "example:OutputDataMessage",
    }
)
# Everything managed together
```

## Migration Path

### Plugin Developer Migration

1. **Define Message Classes**: Create OWAMessage subclasses for custom data types
2. **Update PluginSpec**: Add messages section to existing PluginSpec
3. **Update Dependencies**: Ensure mcap-owa-support version supports dynamic registration
4. **Test Integration**: Verify message types work in OWAMcap files

### User Migration

No changes required for users - message types become automatically available when plugins are installed.

Backwards Compatibility
=======================

This OEP is fully backward compatible:

**No Breaking Changes:**
- Existing plugins continue to work without modification
- Message registration is optional - plugins without messages section work unchanged
- Existing OWAMcap files remain compatible
- All existing APIs maintain their current behavior

**Additive Changes:**
- New `messages` field in PluginSpec (optional)
- Enhanced CLI commands (new functionality)
- Automatic message discovery (transparent to users)

Implementation Requirements
===========================

## Core Components

1. **PluginSpec Enhancement**: Add optional `messages` field with validation
2. **Message Discovery**: Extend plugin discovery to collect message definitions
3. **Registry Integration**: Create message registry similar to component registries
4. **mcap-owa-support Updates**: Automatic message type registration
5. **CLI Enhancements**: New commands for message discovery and validation

## Validation Requirements

- Message type uniqueness across all installed plugins
- Proper OWAMessage inheritance validation
- Schema compatibility checking
- Circular dependency detection

## Testing Requirements

- Message registration and discovery tests
- OWAMcap integration tests with custom messages
- CLI functionality tests
- Plugin installation/uninstallation tests
- Cross-plugin message compatibility tests

Rejection Rationale
===================

This OEP has been rejected in favor of OEP-6, which proposes a cleaner separation of concerns by creating a dedicated `owa-msgs` package. The key issues with the OEP-5 approach include:

1. **Tight Coupling**: Embedding message definitions within the EnvPlugin system creates unnecessary coupling between data schemas and runtime components.

2. **Package Management Complexity**: Having message definitions scattered across multiple plugin packages makes it difficult to manage message schema evolution and compatibility.

3. **Discovery Overhead**: Dynamic message discovery from plugins adds runtime overhead and complexity to the mcap-owa-support package.

4. **Namespace Pollution**: Plugin namespaces become overloaded with both component and message concerns, reducing clarity.

5. **Deployment Complexity**: Users who only need message definitions for data processing would be forced to install entire plugin packages with their runtime dependencies.

The OEP-6 approach addresses these concerns by creating a dedicated message package with clear separation of concerns, better package management, and more efficient discovery mechanisms.

Open Issues
===========

1. **Message Versioning**: How should message schema evolution be handled?
2. **Cross-Plugin Dependencies**: Should plugins be able to reference messages from other plugins?
3. **Schema Validation**: What level of runtime schema validation should be enforced?
4. **Performance Impact**: What is the overhead of dynamic message discovery?

References
==========

- OEP-3: Entry Points-Based Plugin Discovery and Unified Component Naming
- OWAMcap Format Specification: docs/data/data_format_v2.md
- mcap-owa-support Package Documentation

Copyright
=========

This document is placed in the public domain or under the
CC0-1.0-Universal license, whichever is more permissive.
