OEP: 6
Title: Dedicated OWA Message Package and OWAMcap Profile Specification
Author: Open World Agents Team
Status: Final
Type: Standards Track
Replaces: 5
Created: 10-Jun-2025
Post-History: 10-Jun-2025, 14-Jun-2025

Abstract
========

This OEP establishes a dedicated `owa-msgs` package for managing OWA message definitions separately from the EnvPlugin system, addressing architectural issues with the previous approach of embedding message schemas within individual plugins. The proposal formalizes OWAMcap as a specification for MCAP files with the `owa` profile and introduces a message registry system using Python entry points for automatic discovery. This design provides better separation of concerns, simplified package management, improved performance, and enables extensible custom message registration while maintaining full backward compatibility with existing code and MCAP files.

Motivation
==========

The current approach of embedding message definitions within individual EnvPlugins creates several architectural and practical problems:

1. **Tight Coupling**: Message schemas are unnecessarily coupled with runtime plugin components, making it difficult to use messages independently of plugin functionality.

2. **Package Management Complexity**: Users who only need message definitions for data processing must install entire plugin packages with their runtime dependencies, creating unnecessary bloat.

3. **Discovery Overhead**: Dynamic message discovery from multiple plugin packages adds runtime complexity and performance overhead to the mcap-owa-support package.

4. **Namespace Confusion**: Plugin namespaces become overloaded with both component and message concerns, reducing clarity and making the system harder to understand.

5. **Schema Evolution Challenges**: Managing message schema evolution across multiple plugin packages is complex and error-prone.

6. **Deployment Fragmentation**: Message definitions scattered across multiple packages make it difficult to ensure consistent message schemas across different environments.

A dedicated message package addresses these issues by providing a centralized, well-managed location for OWA message definitions while maintaining extensibility through standardized entry points.

Rationale
=========

The design separates message definitions from runtime components while maintaining extensibility:

**Separation of Concerns**: Message schemas are data contracts that should be independent of runtime implementation details. A dedicated package provides clear boundaries.

**Simplified Package Management**: Users can install only the message definitions they need without pulling in unnecessary runtime dependencies.

**Centralized Schema Management**: A single package provides better control over message schema evolution, versioning, and compatibility.

**Performance Optimization**: Static message registration through entry points eliminates runtime discovery overhead.

**Clear Extensibility Model**: The entry point system provides a standardized way for third-party packages to register custom message types.

**Better Documentation**: Centralized message definitions enable comprehensive documentation and schema browsing tools.

Specification
=============

## OWA Messages Package

### Package Structure

The `owa-msgs` package provides the core message definitions for the OWA ecosystem. The package is developed within the open-world-agents repository (`projects/owa-msgs/`) and is added as a dependency to both `owa-core` and `owa` packages to ensure message definitions are automatically available:

```
owa-msgs/
├── pyproject.toml
└── owa/
    └── msgs/
        └── desktop/
            ├── keyboard.py
            ├── mouse.py
            ├── screen.py
            └── window.py
```

**Note**: The `owa.msgs` namespace uses implicit namespace packaging (PEP 420) and deliberately omits `__init__.py` files. This allows multiple packages to contribute to the same namespace without conflicts, enabling third-party packages to extend the `owa.msgs` namespace with their own message definitions.

The package contains only concrete message definitions organized by domain. Base classes (`BaseMessage`, `OWAMessage`) remain in `owa.core.message` to avoid circular dependencies and maintain clear separation between the core framework and message definitions.

### Core Message Definitions

Standard OWA message types are organized by domain:

```python
# owa/msgs/desktop/keyboard.py
from owa.core.message import OWAMessage

class KeyboardEvent(OWAMessage):
    _type = "desktop/KeyboardEvent"
    event_type: str  # "press", "release"
    vk: int
    timestamp: int

class KeyboardState(OWAMessage):
    _type = "desktop/KeyboardState"
    buttons: list[int]
    timestamp: int
```

### Entry Point Registration

The package registers message types through entry points:

```toml
# pyproject.toml
[project.entry-points."owa.msgs"]
"desktop/KeyboardEvent" = "owa.msgs.desktop.keyboard:KeyboardEvent"
"desktop/KeyboardState" = "owa.msgs.desktop.keyboard:KeyboardState"
"desktop/MouseEvent" = "owa.msgs.desktop.mouse:MouseEvent"
"desktop/MouseState" = "owa.msgs.desktop.mouse:MouseState"
"desktop/ScreenCaptured" = "owa.msgs.desktop.screen:ScreenCaptured"
"desktop/WindowInfo" = "owa.msgs.desktop.window:WindowInfo"
```

## OWAMcap Profile Specification

### Formal Definition

OWAMcap is defined as:
- **Base Format**: Standard MCAP container format
- **Profile**: `owa` designation in MCAP metadata
- **Schema Encoding**: JSON Schema
- **Message Interface**: All messages must implement `BaseMessage` from `owa.core.message`
- **Standard Messages**: Core message types from `owa-msgs` package

### Profile Metadata

OWAMcap files must include the following metadata:

```python
# MCAP profile metadata
profile = "owa"
library = f"mcap-owa-support {version}; mcap {mcap_version}"
```

### Automatic Message Discovery

The `owa.core` package provides automatic message discovery through entry points using a `MessageRegistry` class. The registry implements lazy loading and provides dict-like access to message classes:

```python
from owa.core import MESSAGES
KeyboardEvent = MESSAGES['desktop/KeyboardEvent']
```

The registry automatically discovers all message types registered through the `owa.msgs` entry point group and validates that they inherit from `BaseMessage`. See `projects/owa-core/owa/core/messages.py` for the complete implementation.

### Usage Examples

```python
# Import the global message registry
from owa.core import MESSAGES

# Access message classes directly
KeyboardEvent = MESSAGES['desktop/KeyboardEvent']
MouseEvent = MESSAGES['desktop/MouseEvent']

# Check if a message type exists
if 'desktop/KeyboardEvent' in MESSAGES:
    print("KeyboardEvent is available")

# List all available message types
for message_type in MESSAGES.keys():
    print(f"Available: {message_type}")

# Create message instances
event = MESSAGES['desktop/KeyboardEvent'](
    event_type="press",
    vk=65,
    timestamp=1234567890
)
```

## Custom Message Registration

### Third-Party Package Integration

External packages can register custom message types:

```toml
# third-party-package/pyproject.toml
[project.entry-points."owa.msgs"]
"sensors/TemperatureReading" = "custom_sensors.messages:TemperatureReading"
"sensors/HumidityReading" = "custom_sensors.messages:HumidityReading"
```

### Message Type Naming Convention

- **Format**: `domain/MessageType`
- **Domain**: Logical grouping (e.g., `desktop`, `sensors`, `system`)
- **MessageType**: PascalCase message name
- **Examples**: `desktop/KeyboardEvent`, `desktop/WindowInfo`, `sensors/TemperatureReading`

**Note**: The simplified naming convention removes package prefixes to avoid redundancy and improve readability. All core OWA messages use domain-based naming, and third-party packages should follow the same pattern.

## Integration with mcap-owa-support

### Enhanced Message Discovery

The mcap-owa-support package integrates with the `owa.core.MESSAGES` registry to automatically discover and use registered message types. This enables seamless reading and writing of OWAMcap files with automatic message resolution.

### Automatic Schema Registration

Message schemas are automatically available in OWAMcap operations through the registry integration. The decoder can automatically resolve message types from domain-based names and fall back to dictionary decoding when message classes are not available.

### Backward Compatibility

Existing import patterns continue to work alongside the new registry. Both traditional direct imports and registry-based access provide access to the same message classes, ensuring compatibility with existing code.

Backwards Compatibility
=======================

This OEP maintains full backward compatibility with existing OWA installations:

**No Breaking Changes:**
- Existing OWAMcap files remain fully compatible and can be read without modification
- Current mcap-owa-support APIs continue to work unchanged
- Existing message classes continue to function with their original import paths
- All existing code continues to work without requiring immediate updates

**Migration Path:**
- Core OWA message types are available in the new `owa-msgs` package
- Existing imports are maintained through compatibility shims in `owa.env.desktop.msg`
- Users can migrate gradually at their own pace
- Legacy MCAP files can be converted using the `owl mcap convert-legacy` command

**Additive Changes:**
- New entry point-based message discovery system (optional to use)
- Enhanced CLI tools for message browsing and validation
- Improved documentation and schema tools
- Global `MESSAGES` registry for dynamic message access

Implementation Requirements
===========================

## Core Components

1. **owa-msgs Package**: Create dedicated package with core message definitions
2. **Entry Point System**: Implement message registration through entry points
3. **MESSAGES Registry**: Add automatic message discovery to `owa.core` with global `MESSAGES` registry
4. **Registry Integration**: Update mcap-owa-support to use the `owa.core.MESSAGES` registry
5. **CLI Tools**: Add commands for message browsing and validation
6. **Documentation**: Comprehensive message schema documentation

## Migration Strategy

1. **Phase 1**: Create owa-msgs package with core message types (developed in `projects/owa-msgs/`)
2. **Phase 2**: Add owa-msgs as dependency to owa-core and owa packages for automatic availability
3. **Phase 3**: Add entry point discovery to mcap-owa-support
4. **Phase 4**: Add compatibility shims for existing imports
5. **Phase 5**: Update documentation and examples
6. **Phase 6**: Deprecate old message locations (future OEP)

## Validation Requirements

- Message type uniqueness across all registered packages
- Proper BaseMessage interface implementation
- Entry point registration validation
- Schema compatibility checking

Security Implications
=====================

The entry point system introduces minimal security considerations:

- Entry points are resolved at import time, not runtime
- Message classes must still implement the BaseMessage interface
- No arbitrary code execution beyond normal Python import mechanisms
- Package installation follows standard Python security practices

How to Teach This
=================

The new system simplifies the mental model for users:

1. **Message Access**: Import `MESSAGES` from `owa.core` for automatic message discovery
2. **Message Definitions**: Core message types are automatically available through `owa-msgs` dependency (no separate installation needed)
3. **Custom Messages**: Create packages with entry point registration
4. **Usage**: Access messages via `MESSAGES['domain/MessageType']` or traditional imports
5. **Discovery**: Use CLI tools to browse available message types

**Teaching Examples:**

The system follows familiar Python packaging patterns:
- Import the global registry: `from owa.core import MESSAGES`
- Access messages by type: `MESSAGES['domain/MessageType']`
- Use CLI tools to explore: `owl messages list`

This approach provides both convenience and discoverability while maintaining compatibility with traditional import patterns.

Reference Implementation
========================

The reference implementation for this OEP has been completed and is available in the Open World Agents repository. The implementation provides:

## Core Components

1. **Message Registry** - Automatic message discovery through Python entry points (`projects/owa-core/owa/core/messages.py`)
2. **owa-msgs Package** - Dedicated package with core message definitions (`projects/owa-msgs/`, included as dependency in owa-core and owa)
3. **CLI Tools** - Enhanced commands for message management (`projects/owa-cli/owa/cli/messages/`)
4. **mcap-owa-support Integration** - Updated decoder with registry support
5. **Migration Tools** - Legacy compatibility and conversion utilities

## Implementation Status

The implementation is complete and includes comprehensive testing, documentation, and migration tools. All components have been validated for functionality, performance, and backward compatibility.

For detailed implementation information, see the respective package directories in the Open World Agents repository.

Rejected Ideas
==============

**Plugin-Based Message Registration (OEP-5)**: Rejected due to tight coupling between message schemas and runtime components. This approach created unnecessary complexity and deployment overhead.

**Monolithic Message Module**: Considered embedding all message types in owa-core, but rejected because it would prevent third-party message type registration and create a large, unwieldy module.

**Runtime Message Discovery**: Considered dynamic message discovery at runtime, but rejected due to performance overhead and complexity. Entry points provide the same flexibility with better performance.

**Namespace-Free Message Types**: Considered using simple message names without package prefixes, but rejected due to collision risks and lack of clear ownership.

Open Issues
===========

While the core implementation is complete, several areas remain for future consideration:

1. **Message Versioning**: Future OEPs should address how message schema evolution will be handled across package versions, including backward compatibility strategies for schema changes.

2. **Extended Validation**: Consider whether additional runtime schema validation should be enforced beyond the current BaseMessage interface requirements.

3. **Performance Optimization**: Monitor entry point discovery performance as the number of registered message types grows and optimize if necessary.

4. **Third-Party Integration**: Establish best practices and guidelines for third-party packages that want to register custom message types.

5. **Schema Documentation**: Develop automated tools for generating comprehensive schema documentation from message definitions.

References
==========

- MCAP Format Specification: https://mcap.dev/
- Python Entry Points: https://packaging.python.org/specifications/entry-points/
- OWAMcap Format Documentation: docs/data/data_format_v2.md
- BaseMessage Interface: projects/owa-core/owa/core/message.py
- OEP-5: Message Definition Registration in EnvPlugin System (Rejected)

Copyright
=========

This document is placed in the public domain or under the
CC0-1.0-Universal license, whichever is more permissive.
